@@ -13,15 +13,13 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 
-#include "uapi/compel/compel.h"
-
 #include "version.h"
 #include "piegen.h"
 #include "log.h"
 
 #define CFLAGS_DEFAULT_SET					\
 	"-Wstrict-prototypes "					\
-	"-fno-stack-protector -nostdlib -fomit-frame-pointer "
+	"-fno-stack-protector -nostdlib -fomit-frame-pointer -ffreestanding "
 
 #define COMPEL_CFLAGS_PIE	CFLAGS_DEFAULT_SET "-fpie"
 #define COMPEL_CFLAGS_NOPIC	CFLAGS_DEFAULT_SET "-fno-pic"
@@ -52,12 +50,14 @@ static const flags_t flags = {
 #elif defined CONFIG_PPC64
 	.arch		= "ppc64",
 	.cflags		= COMPEL_CFLAGS_PIE,
+#elif defined CONFIG_S390
+	.arch		= "s390",
+	.cflags		= COMPEL_CFLAGS_PIE,
 #else
 #error "CONFIG_<ARCH> not defined, or unsupported ARCH"
 #endif
 };
 
-piegen_opt_t opts = {};
 const char *uninst_root;
 
 static int piegen(void)
@@ -103,7 +103,7 @@ err:
 	if (opts.fout)
 		fclose(opts.fout);
 	if (!ret)
-		printf("%s generated successfully.\n", opts.output_filename);
+		pr_info("%s generated successfully.\n", opts.output_filename);
 	return ret;
 }
 
@@ -181,8 +181,7 @@ static void print_ldflags(bool compat)
 	if (uninst_root) {
 		printf("%s/arch/%s/scripts/compel-pack%s.lds.S\n",
 				uninst_root, flags.arch, compat_str);
-	}
-	else {
+	} else {
 		printf("%s/compel/scripts/compel-pack%s.lds.S\n",
 				LIBEXECDIR, compat_str);
 
@@ -222,8 +221,7 @@ static int print_libs(bool is_static)
 			return 1;
 		}
 		printf("%s/%s\n", uninst_root, STATIC_LIB);
-	}
-	else {
+	} else {
 		printf("%s/%s\n", LIBDIR, (is_static) ? STATIC_LIB : DYN_LIB);
 	}
 
@@ -255,8 +253,7 @@ static char *gen_prefix(const char *path)
 	for (i = len - 1; i >= 0; i--) {
 		if (!p1 && path[i] == '.') {
 			p2 = path + i - 1;
-		}
-		else if (!p1 && path[i] == '/') {
+		} else if (!p1 && path[i] == '/') {
 			p1 = path + i + 1;
 			break;
 		}
