@@ -30,6 +30,7 @@
 #include "common/xmalloc.h"
 
 struct cr_options opts;
+char *rpc_cfg_file;
 
 static int count_elements(char **to_count)
 {
@@ -276,6 +277,7 @@ void init_opts(void)
 	opts.empty_ns = 0;
 	opts.status_fd = -1;
 	opts.log_level = DEFAULT_LOGLEVEL;
+	opts.pre_dump_mode = PRE_DUMP_SPLICE;
 }
 
 bool deprecated_ok(char *what)
@@ -516,6 +518,8 @@ int parse_options(int argc, char **argv, bool *usage_error,
 		{ "tls-key",			required_argument,	0, 1095},
 		BOOL_OPT("tls", &opts.tls),
 		{"tls-no-cn-verify",		no_argument,		&opts.tls_no_cn_verify, true},
+		{ "cgroup-yard",		required_argument,	0, 1096 },
+		{ "pre-dump-mode",		required_argument,	0, 1097},
 		{ },
 	};
 
@@ -814,6 +818,17 @@ int parse_options(int argc, char **argv, bool *usage_error,
 		case 1095:
 			SET_CHAR_OPTS(tls_key, optarg);
 			break;
+		case 1096:
+			SET_CHAR_OPTS(cgroup_yard, optarg);
+			break;
+		case 1097:
+			if (!strcmp("read", optarg)) {
+				opts.pre_dump_mode = PRE_DUMP_READ;
+			} else if (strcmp("splice", optarg)) {
+				pr_err("Unable to parse value of --pre-dump-mode\n");
+				return 1;
+			}
+			break;
 		case 'V':
 			pr_msg("Version: %s\n", CRIU_VERSION);
 			if (strcmp(CRIU_GITID, "0"))
@@ -831,15 +846,15 @@ int parse_options(int argc, char **argv, bool *usage_error,
 
 bad_arg:
 	if (idx < 0) /* short option */
-		pr_msg("Error: invalid argument for -%c: %s\n",
+		pr_err("invalid argument for -%c: %s\n",
 				opt, optarg);
 	else /* long option */
-		pr_msg("Error: invalid argument for --%s: %s\n",
+		pr_err("invalid argument for --%s: %s\n",
 				long_opts[idx].name, optarg);
 	return 1;
 }
 
-int check_options()
+int check_options(void)
 {
 	if (opts.tcp_established_ok)
 		pr_info("Will dump/restore TCP connections\n");
