@@ -149,12 +149,18 @@ extern int tcp_init_server(int family, int *port);
 extern int tcp_accept_server(int sock);
 extern int tcp_init_client(int family, char *servIP, unsigned short servPort);
 
+struct sockaddr_un;
+extern int unix_fill_sock_name(struct sockaddr_un *name, char *relFilename);
+
 struct zdtm_tcp_opts {
 	bool reuseaddr;
 	bool reuseport;
 	int flags;
 };
 
+extern const char *test_author;
+extern const char *test_doc;
+
 extern int tcp_init_server_with_opts(int family, int *port, struct zdtm_tcp_opts *opts);
 extern pid_t sys_clone_unified(unsigned long flags, void *child_stack, void *parent_tid,
 			       void *child_tid, unsigned long newtls);
