@@ -44,7 +44,7 @@ int main(int argc, char **argv)
 	memcpy(addr.sun_path, SK_SRV, sizeof(SK_SRV));
 	addrlen = sizeof(addr.sun_family) + sizeof(SK_SRV);
 
-	if (bind(srv, &addr, addrlen)) {
+	if (bind(srv, (struct sockaddr *) &addr, addrlen)) {
 		fail("bind\n");
 		exit(1);
 	}
@@ -58,14 +58,14 @@ int main(int argc, char **argv)
 		memcpy(addr.sun_path, sk_names[i], sizeof(SK_NAME));
 		addrlen = sizeof(addr.sun_family) + sizeof(SK_NAME);
 
-		if (bind(clnt, &addr, addrlen)) {
+		if (bind(clnt, (struct sockaddr *) &addr, addrlen)) {
 			fail("bind\n");
 			exit(1);
 		}
 
 		memcpy(addr.sun_path, SK_SRV, sizeof(SK_SRV));
 		addrlen = sizeof(addr.sun_family) + sizeof(SK_SRV);
-		if (connect(clnt, &addr, addrlen)) {
+		if (connect(clnt, (struct sockaddr *) &addr, addrlen)) {
 			fail("connect\n");
 			exit(1);
 		}
